
* Alternate Count Data Model Chapter
drop _all

set obs 21
scalar mu     = 4
scalar alpha1 = 0.5
scalar alpha2 = 1.5
scalar p1     = 1/(1+alpha1*mu)
scalar p2     = 1/(1+alpha2*mu)
generate y = _n-1
generate double fy0p0 = exp(-mu)*mu^y/exp(lngamma(y+1))
generate double fy0p5 = exp(lngamma(y+1/alpha1)-lngamma(y+1)-lngamma(1/alpha1))*p1^(1/alpha1)*(1-p1)^y
generate double fy1p5 = exp(lngamma(y+1/alpha2)-lngamma(y+1)-lngamma(1/alpha2))*p2^(1/alpha2)*(1-p2)^y
label var fy0p0 "NB-alpha=0.0"
label var fy0p5 "NB-alpha=0.5"
label var fy1p5 "NB-alpha=1.5"
twoway (scatter fy0p0 y, c(l)) (scatter fy0p5 y, c(l)) (scatter fy1p5 y, c(l)), title("Mean=4: Count Data Models") 

drop _all
set seed 2349
set obs 50000
generate x1 = rnormal()
generate x2 = rnormal()
generate xb = -1.0 + 0.5*x1 + 1.5*x2
generate lambda = exp(xb)
generate yp = rpoisson(lambda)

poisson yp x1 x2

poisson yp x1 x2 if y>0, nolog

tpoisson yp x1 x2 if y>0, nolog

tabulate yp if yp <= 15

zip yp x1 x2, inflate(x1 x2) iterate(20) nolog


clear
set obs 50000
set seed 1234
generate x1 = rnormal()      /* x1 ~ N(0,1) */
generate x2 = rnormal()      /* x2 ~ N(0,1) */
generate yp = rpoisson(exp(2.0+0.75*x1-1.25*x2)) /* yp ~ Poisson           */
generate p  = 1/(1+exp((0.2+0.9*x1+0.1*x2)))     /* p=P(B=0) for Bernoulli */
generate yb = runiform() < p                     /* yb ~ Bernoulli(1-p)    */
generate yz = yb*yp                              /* yz ~ ZIP               */
generate yzero = yz == 0                         /* yzero = 1 if yz=0      */

zip yz x1 x2, inflate(x1 x2) nolog

use rwm1984, clear
zigp docvis outwork female age married, inflate(female age) zip nolog
estat ic

quietly gpoisson docvis outwork female age married
estat ic

*marginalized
clear
set seed 2323423
set obs 1000
generate x1 = runiform() < .5
generate x2 = runiform() < .5
generate x3 = runiform() < .5
generate yc = rpoisson(exp(1+x1-x2))
generate yz = rbinomial(1, 1/(1+exp(1+x1-x3)))
generate y  = yc * (yz!=0)


capture program drop zipllf
program zipllf
    args lnf beta1 beta2
    quietly {
        tempvar pi mu gamma
        generate double `pi' = exp(`beta2')
        generate double `gamma' = `pi'/(1+`pi')
        generate double `mu' = exp(`beta1')
        replace `lnf' = cond($ML_y1==0,                ///
            ln(`pi'/(1+`pi')+1/(1+`pi')*exp(-`mu')),   ///
            ln(1/(1+`pi')) -`mu' + $ML_y1 * log(`mu')  ///
            - lngamma($ML_y1 + 1))
    }
end
ml model lf zipllf (y:y=x1 x2) (logit: x1 x3)
ml maximize, search(on) nolog title(Zero-inflated Poisson)


capture program drop zimpllf
program zimpllf
    args lnf beta1 beta2
    quietly {
        tempvar pi mu gamma
        generate double `pi' = exp(`beta2')
        generate double `gamma' = `pi'/(1+`pi')
        generate double `mu' = exp(`beta1')/(1-`gamma')
        replace `lnf' = cond($ML_y1==0,                ///
            ln(`pi'/(1+`pi')+1/(1+`pi')*exp(-`mu')),   ///
            ln(1/(1+`pi')) -`mu' + $ML_y1 * log(`mu')  ///
            - lngamma($ML_y1 + 1))
    }
end
ml model lf zimpllf (margin:y=x1 x2) (logit: x1 x3)
ml maximize, search(on) nolog title(Zero-inflated marginalized Poisson)


*truncated 

use medpar, clear
generate byte type = type1 + 2*type2 + 3*type3
label define tlab 1 "elective" 2 "urgent" 3 "emergency"
label values type tlab
trncregress los white hmo i.type, dist(poisson) ltrunc(0) nolog eform
estimates store trnc_poi

trncregress los white hmo i.type, dist(gpoisson) ltrunc(0) nolog eform
estimates store trnc_gpo

trncregress los white hmo i.type, dist(pig) nolog ltrunc(0) vce(robust)
estimates store trnc_pig

trncregress los white hmo i.type if los>0 & los<19, dist(gpoisson) ltrunc(0) rtrunc(19) nolog eform vce(robust)
estimates store trnc_poirt
estimates stats trnc*


capture program drop HurdPoissLogitLL
clear
set obs 50000
set seed 1234
generate x1 = rnormal()
generate x2 = rnormal() 
// Generate Poissons in a loop; keep replacing 
// values until there are no zeros
generate yp = 0
count if yp==0
while r(N) {
        quietly {
                replace yp = rpoisson(exp(2.0+0.75*x1-1.25*x2)) if yp==0
                count if yp==0
        }
}
generate p  = 1/(1+exp((0.2+0.9*x1+0.1*x2)))   // Binary p
generate yb = runiform() < p   // Zero outcomes               
generate yz = yb*yp            // Hurdle outcome to model
generate yzero = (yz == 0)     // Indicator of zeros
program HurdPoissLogitLL
    args lnf beta1 beta2
    quietly {
        tempvar pi mu
        generate double `mu' = exp(`beta1')
        generate double `pi' = exp(`beta2')
        replace `lnf' = cond($ML_y1==0,              ///
            ln(`pi'/(1+`pi')),                       ///
            ln(1/(1+`pi')) -`mu' + $ML_y1 * `beta1'  ///
            - lngamma($ML_y1 + 1) - ln(1 - exp(-`mu')))
    }
end
ml model lf HurdPoissLogitLL (yz:yz=x1 x2) (logit:yzero=x1 x2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression)

logit yzero x1 x2, nolog

tpoisson yz x1 x2 if yz>0, nolog

ml model lf HurdPoissLogitLL (yz:yz=x1 x2) (logit:yzero=x1 x2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression)
predict xb1 if yz>0, eq(#1)           /* zero-truncated Poisson prediction */
predict xb2, eq(#2)                   /* logit prediction                  */
generate mu1 = exp(xb1)               /* predicted ZTP count               */
generate mu2 = exp(xb2)/(1+exp(xb2))  /* predicted Pr(yz=0)                */

label define zlab 0 "yz != zero" 1 "yz = zero"
label values yzero zlab
table yzero, c(freq mean mu2)        // Show mean predicted probabilities 
summarize mu1 yz if yz > 0     // Show mean predicted and observed counts


hplogit yz x1 x2, nolog

clear
set obs 50000
set seed 1234
generate x1 = rnormal()       /* x1 ~ N(0,1)    */
generate x2 = rnormal()       /* x2 ~ N(0,1)    */
generate z1 = runiform() < .2 /* z1 ~ Ber(0.20) */
generate z2 = runiform() < .4 /* z2 ~ Ber(0.40) */
generate yp = 0
count if yp==0
while r(N) {
        quietly {
                replace yp = rpoisson(exp(2.0+0.75*x1-1.25*x2)) if yp==0
                count if yp==0
        }
}                                                /* yp ~ ZTP                  */
generate p  = 1/(1+exp((-1.0+0.9*z1+0.1*z2)))    /* p=P(B=0) for Bernoulli    */
generate yb = runiform() < p                     /* yb ~ Bernoulli(p)         */
generate yh = yp*yb                              /* yh ~ Poisson-logit hurdle */

generate yhzero = yh==0
ml model lf HurdPoissLogitLL (yh:yh=x1 x2) (logit:yhzero=z1 z2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression)


* Negbin(p)
clear
set seed 2942
set obs  50000
generate x1 = rnormal()
generate x2 = rnormal()
generate expxb1 = exp(1.00 - 1.00*x1 + 1.25*x2)
generate ap1    = 0.33*expxb1
generate expxb2 = exp(1.00 + 1.00*x1 + 1.00*x2)
generate ap2    = 0.25
generate ynb1 = rpoisson(rgamma(ap1, 1/ap1) * expxb1) // ynb1 ~ NB(1) 
generate ynb2 = rpoisson(rgamma(1/ap2, ap2) * expxb2) // ynb2 ~ NB(2)

capture program drop NegbinpLL
program NegbinpLL
    args lnf xb P lntheta
    quietly {
        tempvar g r lambda theta
        generate double `theta'  = exp(`lntheta')
        generate double `lambda' = exp(`xb') 
        generate double `g' = `theta'*`lambda'^(2-`P')
        generate double `r' = `g'/(`g'+`lambda')
        replace `lnf' = lngamma($ML_y1+`g') - lngamma(`g') ///
            - lngamma($ML_y1+1) + `g'*ln(`r') + $ML_y1*ln(1-`r')
    }
end



quietly nbreg ynb1 x1 x2, disp(cons)  // Run NB(1) model
matrix b = e(b)                       // Get coefficient vector: beta,lntheta
local nc = colsof(b)
local ncp1 = `nc'+1                   // Start vals NB(P=1): beta,1,1/lntheta
local lntheta = 1/b[1,`nc']
matrix b[1,`nc'] = 1                  // Insert 1 for power
matrix b = b,`lntheta'                // Add lntheta to end of vector
ml model lf NegbinpLL (ynb1:ynb1 = x1 x2) (P:) (lntheta:)
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta"))

quietly nbreg ynb2 x1 x2     // Run NB(2) model
matrix b = e(b)              // Get coefficient vector: beta,lntheta
local nc = colsof(b)
local ncp1 = `nc'+1          // Start vals NB(P=2): beta,2,lntheta
local lntheta = b[1,`nc']
matrix b[1,`nc'] = 2         // Insert 2 for power
matrix b = b,`lntheta'       // Add lntheta to end of vector
ml model lf NegbinpLL (ynb2:ynb2 = x1 x2) (P:) (lntheta:)
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta"))


use rwm5panel, clear
quietly nbreg docvis outwork age female edlevel3 edlevel4, dispersion(constant)
estimates store A1
quietly nbreg docvis outwork age female edlevel3 edlevel4
estimates store A2
matrix b = e(b)
local nc = colsof(b)
local ncp1 = `nc'+1
local lntheta = 1/b[1,`nc']
matrix b[1,`nc'] = 2
matrix b = b,`lntheta'
ml model lf NegbinpLL (docvis:docvis = outwork age female edlevel3 edlevel4) (P:) (lntheta:)
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta")) irr
estimates store B

estimates restore A1
ml display, irr noheader
lrtest A1 B, force

estimates restore A2
ml display, irr noheader
lrtest A2 B, force

estimates restore B
test [P]_cons=1
test [P]_cons=2

use medpar, clear
nbreg los hmo white type2 type3 died, nolog

gnbreg los hmo white type2 type3 died, lnalpha(type2 type3 died) nolog


glm los hmo white type2 type3 died, family(nbinomial .4339959) notable nolog

glm los hmo white type2 type3 died, family(poisson) notable nolog

gnbreg los hmo white type2 type3, lnalpha(died) irr nolog

gnbreg 

gpoisson los hmo white type2 type3, irr nolog

use doll, clear
glm deaths smokes a2-a5, exposure(pyears) family(poi) irls nolog 
predict double pearson, pearson
predict double yhatp

use doll, clear
gpoisson deaths smokes a2-a5, exposure(pyears) irr nolog 


use azdrg112, clear
gpoisson los male age75 emergency, nolog
estat ic

quietly poisson los male age75 emergency
estat ic

use azdrg112, clear
pigreg los male age75 emergency, nolog
estat ic

pigreg, irr

use medparc, clear
describe

tabulate cenx

list los cenx if cenx == -1

tabulate los if los > 60

tabulate los if los > 20 & los < 30

capture program drop CensPoisson
generate cens = cond(cenx==-1,-1,los)
program CensPoisson
    args lnf xb
    quietly {
        tempvar p1 p2
        generate double `p1' = cond($ML_y1==0,1,gammap($ML_y1,exp(`xb')))
        generate double `p2' = cond($ML_y2==-1,0,gammap($ML_y2+1,exp(`xb')))
        replace `lnf' = ln(`p1'-`p2') if $ML_samp
    }
end
ml model lf CensPoisson (los:los cens=hmo white type2 type3)
ml maximize, search(on) nolog title(Censored Poisson Regression) irr
estat ic

capture drop covpattern
egen covpattern = group(hmo white type2 type3)
table covpattern if e(sample), contents(freq mean los sd los)


capture program drop CensNegBin
program CensNegBin
    args todo b lnf 
    quietly {
        tempvar xb lnalpha r p ll
        mleval `xb'      = `b', eq(1)
        mleval `lnalpha' = `b', eq(2)
        generate double `r' = exp(-`lnalpha')
        generate double `p' = 1/(1+exp(-`xb'-`lnalpha'))
        tempvar p1 p2
        generate double `p1' = cond($ML_y1==0,1,ibeta($ML_y1,`r',`p'))
        generate double `p2' = cond($ML_y2==-1,0,ibeta($ML_y2+1,`r',`p'))
        replace `lnf' =  ln(`p1'-`p2')
    }
end
ml model gf0 CensNegBin (los:los cens=hmo white type2 type3) (lnalpha:)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr
estat ic

ml model gf0 CensNegBin (los:los los=hmo white type2 type3) (lnalpha:)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr 
estat ic

ml model gf0 CensNegBin (los:los cens=hmo white type2 type3) (lnalpha:), ///
    vce(cluster provnum)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr
estat ic

replace cens  = -1 if los>=30  // Censored at 30
generate los30 = min(los,30)   // Dataset-censor value
ml model gf0 CensNegBin (los:los30 cens=hmo white type2 type3) (lnalpha:), ///
    vce(cluster provnum)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr 

use azdrg112, clear
fmm 3, noheader nolog: poisson los male emergency age75
estat ic

estat lcprob


predict fmmhat, marginal
summarize fmmhat

clear
set obs 75000
set seed 2342
generate n1 = rnormal()
generate n2 = rnormal()
generate b1 = runiform() < .4
generate b2 = runiform() < .6
generate expxb1 = exp(2.00 + 1.50*n1 + -0.5*n2 + 3.00*b1 - 4.0*b2)
generate ap1    = 0.66
generate expxb2 = exp(1.00 + 1.00*n1 + 1.00*b1 - 2.0*b2)
generate ap2    = 0.25
generate expxb3 = exp(1.00 - 1.00*n1 + 1.25*n2)
generate ap3    = 0.33
generate ynb1 = rpoisson(rgamma(1/ap1, ap1) * expxb1)   // ynb1 ~ NB(2) 
generate ynb2 = rpoisson(rgamma(1/ap2, ap2) * expxb2)   // ynb2 ~ NB(2)
generate ynb3 = rpoisson(rgamma(1/ap3, ap3) * expxb3)   // ynb3 ~ NB(2)
generate mix = runiform()
generate yfmm = (mix>0.40)*ynb1 + (mix<0.25)*ynb2 + (mix>0.25 & mix<0.40)*ynb3 

nbreg yfmm n1 n2 b1 b2, nolog 

fmm 3, noheader nolog: nbreg yfmm n1 n2 b1 b2
estat ic

fmm, noheader nolog: (nbreg yfmm n1 n2) (nbreg yfmm n1 b1 b2) ///
                     (nbreg yfmm n1 n2 b1 b2) ///


use rwm5panel, clear
nbregp docvis outwork age female edlevel3 edlevel4, irr nolog


clear
set seed 29342
set obs 1000
generate x1 = runiform() < .5
generate x2 = runiform() < .5
generate p  = exp(-x1+2*x2)/(1+exp(-x1+2*x2))
generate y1 = rpoisson(exp(1+x1+2*x2))
generate y2 = 3*rpoisson(exp(1+x1+2*x2-ln(3)))
generate y  = cond(runiform()<p,y2,y1)
histogram y, width(1)

poisson y x1 x2, nolog
forvalues k=0/100 {
        predict predp`k', pr(`k')
}

heapr y x1 x2, heap(3) nolog
forvalues k=0/100 {
        predict predh`k', pr(`k')
}

capture program drop GetHeapPred
program GetHeapPred
        args x1 x2 ll uu
        * The arguments are:
        * x1,x2 = the specific values of the covariates
        * ll,uu = the specific range (min,max) for which to
        *         to generate predictions.
        * assumed but not enforced: ll>=0 and uu<=100
        quietly {
                count if x1==`x1' & x2==`x2'
                local Nden = `r(N)'
                tempvar xx yd yp yh
                foreach var in xx yd yp yh {
                        generate double ``var'' = .
                        replace `xx' = _n-1
                }
                forvalues k=0/100 {
                        count if y==`k' & x1==`x1' & x2==`x2'
                        replace `yd' = `r(N)'/`Nden' if _n==`k'+1
                        summ predp`k' if x1==`x1' & x2==`x2'
                        replace `yp' = `r(mean)' if _n==`k'+1
                        summ predh`k' if x1==`x1' & x2==`x2'
                        replace `yh' = `r(mean)' if _n==`k'+1
                }
                label var `yd' "Data"
                label var `yp' "Poisson"
                label var `yh' "Heap"
                label var `xx' "x"
                twoway (bar `yd' `xx',  bstyle(outline)) /// 
                    (connected `yp' `xx', msymbol(circle_hollow) msize(small))   ///
                    (scatter `yh' `xx', msymbol(smtriangle_hollow) msize(small)) ///
                    in `ll'/`uu', scheme(s1mono) legend(rows(1))
        }
end
GetHeapPred 0 0 1 13
graph save pred00, replace
GetHeapPred 0 1 1 41
graph save pred01, replace
GetHeapPred 1 0 1 21
graph save pred10, replace
GetHeapPred 1 1 40 81
graph save pred11, replace

graph combine pred00.gph pred01.gph pred10.gph pred11.gph, rows(2)


use rwm1984, clear
nbregf docvis edlevel4 age hh, eform nolog

nbregw docvis edlevel4 age hh, eform nolog

use medpar, clear
quietly qcount los hmo white type2 type3 died, quantile(.50) repetition(1000)
qcount

quietly qcount los hmo white type2 type3 died, quantile(.25) repetition(1000)
qcount
quietly qcount los hmo white type2 type3 died, quantile(.75) repetition(1000)
qcount

