* Nonlogit Chapter
drop _all

range eta -3 3 100
generate logit  = exp(eta)/(1+exp(eta))
generate probit = normal(eta)
generate clog   = 1-exp(-exp(eta))
generate logl   = exp(-exp(-eta))
capture program drop doit1
program doit1
    version 6.0
    graph logit probit eta, ylab(0,.25,.5,.75,1) xlab(-3,-2,-1,0,1,2,3) /*
        */ saving(sigmoid, replace) s(o.) c(.l) t1(" ") t2("logit(symbols)    probit(line)")
end
capture program drop doit2
program doit2
    version 6.0
    graph clog logl eta, ylab(0,.25,.5,.75,1) xlab(-3,-2,-1,0,1,2,3) /*
        */ saving(sigmoid2, replace) s(o.) c(.l) t1(" ") t2("cloglog(symbols)    loglog(line)")
end
capture program drop doit3
program doit3
	version 6.0
	graph logit probit eta eta if eta>=-1 & eta<=2, ylab(0,.25,.5,.75,1) /*
		*/ xlab(-1,0,1,2) saving(sigmoid3, replace) s(op.) c(..l) t1(" ") /*
		*/ t2("logit(o)    probit(+)   identity(line)") yline(0,1)
end
doit1
doit2
doit3



drop _all
set obs 101
generate eta = -3 + 6*(_n-1)/100
generate logit   = exp(eta)/(1+exp(eta))
generate probit  = normal(eta)
generate loglog  = exp(-exp(-eta))
generate cloglog = 1-exp(-exp(eta)) 
generate identity= eta 
twoway (scatter logit eta, c(l) s(s) xlabel(-3(1)3)) (scatter probit eta, c(l) s(o) ylabel(0(.25)1))

twoway (scatter loglog eta, c(l) s(s) xlabel(-3(1)3)) (scatter cloglog eta, c(l) s(o) ylabel(0(.25)1))

keep if eta >= -1
keep if eta <= 2

twoway  (scatter logit eta , c(l) s(s) xlabel(-1(1)2) yline(1) ylabel(0(.25)1)) (scatter probit eta, c(l) s(o) ylabel(0(.25)1) yline(0)) (scatter identity eta, ylabel(0(.25)1) msymbol(none) c(l)), legend(rows(1))

use heart01, clear
glm death anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial) link(probit) nolog

use warsaw
glm menarche age, family(binomial total) link(probit) nolog

generate proportion = menarche/total
label var proportion "Observed prop."
glm menarche age, family(binomial total) nolog
predict double logithat
replace logithat = logithat/total
label var logithat "Predicted prop. (logit)"
predict double probithat
replace probithat = probithat/total
label var probithat "Predicted prop. (probit)"
twoway (scatter probithat age, s(s)) (scatter logithat age, s(o)) (scatter proportion age, s(x)), legend(rows(1))

use heart01, clear
quietly glm death anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial) ///
	link(probit) nolog
tabulate death if e(sample)

glm death anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial) link(loglog) nolog

use warsaw
glm menarche age, family(binomial total) link(logit) notable nolog
glm menarche age, family(binomial total) link(probit) notable nolog
glm menarche age, family(binomial total) link(loglog) notable nolog
glm menarche age, family(binomial total) link(cloglog) notable nolog

clear
drop _all
input trt rep depth damage examine tpaper
1 1 1 120 187 1
1 2 1 145 181 1
1 3 1 123 184 1
1 1 2  60 184 2
1 2 2  85 191 2
1 3 2 113 171 2
1 1 3  35 179 3
1 2 3  23 181 3
1 3 3  33 179 3
1 1 4   5 178 4
1 2 4  40 184 4
1 3 4  18 171 4
1 1 5  66 182 5
1 2 5 104 186 5
1 3 5 106 181 5
2 1 1  97 187 6
2 2 1 128 173 6
2 3 1 112 179 6
2 1 2  85 186 7
2 2 2  95 167 7
2 3 2 137 194 7
2 1 3  72 190 8
2 2 3  89 184 8
2 3 3  92 192 8
2 1 4  49 188 9
2 2 4  50 187 9
2 3 4  65 186 9
2 1 5 132 179 10
2 2 5 122 182 10
2 3 5 120 179 10
3 1 1 138 187 11
3 2 1 169 184 11
3 3 1 163 175 11
3 1 2 156 176 11
3 2 2 169 190 11
3 3 2 159 175 11
end

tabulate tpaper, gen(T)
tabulate rep, gen(R)


generate prop = damage/examine
label var tpaper "Treatment"
label var prop   "Observed proportion"
egen mprop = mean(prop), by(tpaper)
twoway scatter prop tpaper, xlabel(1(1)11)

local sv = 0 

* This is the exact answer quoted in Famoye's paper
glm damage R2 R3 T1 T2 T3 T4 T5 T6 T7 T8 T9 T10, family(binomial examine) link(cloglog) 
predict double glm
replace glm = glm/examine
egen mglm = mean(glm), by(tpaper)

matrix b0 = e(b)
local ll0 = `e(ll)'
matrix z0 = (0) 
matrix b0 = b0, z0

capture program drop gbinreg_ll
program gbinreg_ll
    version 15
    args todo b lnf 
    tempvar  eta pi 
    tempname phi
    mleval `eta' = `b', eq(1)
    mleval `phi' = `b', eq(2) scalar
    local y  "$ML_y1"   /* Dep var name is in global macro by ml  */
    local M  "examine"  /* Specify directly -- problem specific   */
    generate double `pi'    = 1-exp(-exp(`eta'))  /* Cloglog link here */
    mlsum `lnf' = log(`M') + `y'*log(`pi') +              /*
        */ (`M'+`phi'*`y'-`y')*log(1+`phi'*`pi'-`pi') - /*
        */ (`M'+`phi'*`y')*log(1+`phi'*`pi') - log(`M'+`phi'*`y') + /*
        */ lngamma(`M'+`phi'*`y'+1) - lngamma(`y'+1) -  /*
        */ lngamma(`M'+`phi'*`y'-`y'+1)
end

ml model d0 gbinreg_ll (gbr: damage = R2 R3 T1 T2 T3 T4 T5 T6 T7 T8 T9 T10) (phi:), search(off) init(b0, copy) max iter(50)
ml display, title(Generalized binomial regression)

gbin damage R2 R3 T1 T2 T3 T4 T5 T6 T7 T8 T9 T10, n(examine) link(cloglog) nolog

predict double xb
generate double gb = 1-exp(-exp(xb))
egen mgbr = mean(gb), by(tpaper)

list mprop mglm mgbr

use titanic_altered, clear 
betabin survive adult male class2 class3, n(N) link(cloglog) eform nolog

set seed 23942
drop _all
set obs 5000
gen z1 = rbinomial(1,0.5)
gen z2 = rbinomial(1,0.5)
gen zg = -0.5 + 0.25*z1 - 0.25*z2
gen z  = rbinomial(1, 1-exp(-exp(-zg)))   // link(log-log)
gen x1 = rbinomial(1, 0.5)
gen xb = -1+0.5*x1
gen n  = floor(8*runiform()) + 1
local sigma = 0.33
gen p  = exp(xb)/(1+exp(xb))  // link(logit)

	// y is beta-binomial, yo is zero-inflated beta-binomial
gen y = rbinomial(n, rbeta((1/`sigma')*p, (1/`sigma')*(1-p)))
gen yo = y*z

zibbin yo x1, inflate(z1 z2) n(n) link(logit) ilink(loglog) zib nolog

capture erase sigmoid*.gph
